@@ -1,11 +1,16 @@
 import logging
+import subprocess
 import urwid
 
 from concurrent.futures import ThreadPoolExecutor
+from typing import NamedTuple, Optional
+from datetime import datetime, timezone
 
 from toot import api, config, __version__, settings
-from toot.console import get_default_visibility
+from toot import App, User
+from toot.cli import get_default_visibility
 from toot.exceptions import ApiError
+from toot.utils.datetime import parse_datetime
 
 from .compose import StatusComposer
 from .constants import PALETTE
@@ -14,7 +19,8 @@ from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource, StatusL
 from .overlays import StatusDeleteConfirmation, Account
 from .poll import Poll
 from .timeline import Timeline
-from .utils import get_max_toot_chars, parse_content_links, show_media, copy_to_clipboard
+from .utils import get_max_toot_chars, parse_content_links, copy_to_clipboard
+from .widgets import ModalBox
 
 logger = logging.getLogger(__name__)
 
@@ -24,6 +30,13 @@ urwid.set_encoding('UTF-8')
 DEFAULT_MAX_TOOT_CHARS = 500
 
 
+class TuiOptions(NamedTuple):
+    colors: int
+    media_viewer: Optional[str]
+    relative_datetimes: bool
+    default_visibility: Optional[bool]
+
+
 class Header(urwid.WidgetWrap):
     def __init__(self, app, user):
         self.app = app
@@ -79,9 +92,11 @@ class TUI(urwid.Frame):
     screen: urwid.BaseScreen
 
     @staticmethod
-    def create(app, user, args):
+    def create(app: App, user: User, args: TuiOptions):
         """Factory method, sets up TUI and an event loop."""
-        screen = TUI.create_screen(args)
+        screen = urwid.raw_display.Screen()
+        screen.set_terminal_properties(args.colors)
+
         tui = TUI(app, user, screen, args)
 
         palette = PALETTE.copy()
@@ -100,23 +115,11 @@ class TUI(urwid.Frame):
 
         return tui
 
-    @staticmethod
-    def create_screen(args):
-        screen = urwid.raw_display.Screen()
-
-        # Determine how many colors to use
-        default_colors = 1 if args.no_color else 16
-        colors = settings.get_setting("tui.colors", int, default_colors)
-        logger.debug(f"Setting colors to {colors}")
-        screen.set_terminal_properties(colors)
-
-        return screen
-
-    def __init__(self, app, user, screen, args):
+    def __init__(self, app, user, screen, options: TuiOptions):
         self.app = app
         self.user = user
-        self.args = args
         self.config = config.load_config()
+        self.options = options
 
         self.loop = None  # late init, set in `create`
         self.screen = screen
@@ -137,15 +140,17 @@ class TUI(urwid.Frame):
         self.exception = None
         self.can_translate = False
         self.account = None
+        self.followed_accounts = []
+        self.preferences = {}
 
         super().__init__(self.body, header=self.header, footer=self.footer)
 
     def run(self):
         self.loop.set_alarm_in(0, lambda *args: self.async_load_instance())
-        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_accounts())
-        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_tags())
+        self.loop.set_alarm_in(0, lambda *args: self.async_load_preferences())
         self.loop.set_alarm_in(0, lambda *args: self.async_load_timeline(
             is_initial=True, timeline_name="home"))
+        self.loop.set_alarm_in(0, lambda *args: self.async_load_followed_accounts())
         self.loop.run()
         self.executor.shutdown(wait=False)
 
@@ -173,8 +178,8 @@ class TUI(urwid.Frame):
 
         return urwid.Filler(intro)
 
-    def run_in_thread(self, fn, args=[], kwargs={}, done_callback=None, error_callback=None):
-        """Runs `fn(*args, **kwargs)` asynchronously in a separate thread.
+    def run_in_thread(self, fn, done_callback=None, error_callback=None):
+        """Runs `fn` asynchronously in a separate thread.
 
         On completion calls `done_callback` if `fn` exited cleanly, or
         `error_callback` if an exception was caught. Callback methods are
@@ -198,7 +203,10 @@ class TUI(urwid.Frame):
                 logger.exception(exception)
                 self.loop.set_alarm_in(0, lambda *args: _error_callback(exception))
 
-        future = self.executor.submit(fn, *args, **kwargs)
+        # TODO: replace by `self.loop.event_loop.run_in_executor` at some point
+        # Added in https://github.com/urwid/urwid/issues/575
+        # Not yet released at the time of this comment
+        future = self.loop.event_loop._loop.run_in_executor(self.executor, fn)
         future.add_done_callback(_done)
         return future
 
@@ -249,7 +257,7 @@ class TUI(urwid.Frame):
 
         # This is pretty fast, so it's probably ok to block while context is
         # loaded, can be made async later if needed
-        context = api.context(self.app, self.user, status.original.id)
+        context = api.context(self.app, self.user, status.original.id).json()
         ancestors = [self.make_status(s) for s in context["ancestors"]]
         descendants = [self.make_status(s) for s in context["descendants"]]
         statuses = ancestors + [status] + descendants
@@ -304,7 +312,7 @@ class TUI(urwid.Frame):
         See: https://github.com/mastodon/mastodon/issues/19328
         """
         def _load_instance():
-            return api.get_instance(self.app.base_url)
+            return api.get_instance(self.app.base_url).json()
 
         def _done(instance):
             self.max_toot_chars = get_max_toot_chars(instance, DEFAULT_MAX_TOOT_CHARS)
@@ -324,6 +332,19 @@ class TUI(urwid.Frame):
 
         return self.run_in_thread(_load_instance, done_callback=_done)
 
+    def async_load_preferences(self):
+        """
+        Attempt to update user preferences from instance.
+        https://docs.joinmastodon.org/methods/preferences/
+        """
+        def _load_preferences():
+            return api.get_preferences(self.app, self.user).json()
+
+        def _done(preferences):
+            self.preferences = preferences
+
+        return self.run_in_thread(_load_preferences, done_callback=_done)
+
     def async_load_followed_accounts(self):
         def _load_accounts():
             try:
@@ -339,22 +360,6 @@ class TUI(urwid.Frame):
 
         self.run_in_thread(_load_accounts, done_callback=_done_accounts)
 
-    def async_load_followed_tags(self):
-        def _load_tag_list():
-            try:
-                return api.followed_tags(self.app, self.user)
-            except ApiError:
-                # not supported by all Mastodon servers so fail silently if necessary
-                return []
-
-        def _done_tag_list(tags):
-            if len(tags) > 0:
-                self.followed_tags = [t["name"] for t in tags]
-            else:
-                self.followed_tags = []
-
-        self.run_in_thread(_load_tag_list, done_callback=_done_tag_list)
-
     def refresh_footer(self, timeline):
         """Show status details in footer."""
         status, index, count = timeline.get_focused_status_with_counts()
@@ -414,11 +419,45 @@ class TUI(urwid.Frame):
         def _post(timeline, *args):
             self.post_status(*args)
 
-        composer = StatusComposer(self.max_toot_chars, self.user.username, in_reply_to)
+        # If the user specified --default-visibility, use that; otherwise,
+        # try to use the server-side default visibility.  If that fails, fall
+        # back to get_default_visibility().
+        visibility = (self.options.default_visibility or
+                      self.preferences.get('posting:default:visibility',
+                                           get_default_visibility()))
+
+        composer = StatusComposer(self.max_toot_chars, self.user.username,
+                                  visibility, in_reply_to)
         urwid.connect_signal(composer, "close", _close)
         urwid.connect_signal(composer, "post", _post)
         self.open_overlay(composer, title="Compose status")
 
+    def async_edit(self, status):
+        def _fetch_source():
+            return api.fetch_status_source(self.app, self.user, status.id).json()
+
+        def _done(source):
+            self.close_overlay()
+            self.show_edit(status, source)
+
+        please_wait = ModalBox("Loading status...")
+        self.open_overlay(please_wait)
+
+        self.run_in_thread(_fetch_source, done_callback=_done)
+
+    def show_edit(self, status, source):
+        def _close(*args):
+            self.close_overlay()
+
+        def _edit(timeline, *args):
+            self.edit_status(status, *args)
+
+        composer = StatusComposer(self.max_toot_chars, self.user.username,
+                                  visibility=None, edit=status, source=source)
+        urwid.connect_signal(composer, "close", _close)
+        urwid.connect_signal(composer, "post", _edit)
+        self.open_overlay(composer, title="Edit status")
+
     def show_goto_menu(self):
         user_timelines = self.config.get("timelines", {})
         user_lists = api.get_lists(self.app, self.user) or []
@@ -514,8 +553,20 @@ class TUI(urwid.Frame):
 
     def show_media(self, status):
         urls = [m["url"] for m in status.original.data["media_attachments"]]
-        if urls:
-            show_media(urls)
+        if not urls:
+            return
+
+        media_viewer = self.options.media_viewer
+        if media_viewer:
+            try:
+                subprocess.run([media_viewer] + urls)
+            except FileNotFoundError:
+                self.footer.set_error_message(f"Media viewer not found: '{media_viewer}'")
+            except Exception as ex:
+                self.exception = ex
+                self.footer.set_error_message("Failed invoking media viewer. Press X to see exception.")
+        else:
+            self.footer.set_error_message("Media viewer not configured")
 
     def show_context_menu(self, status):
         # TODO: show context menu
@@ -538,10 +589,15 @@ class TUI(urwid.Frame):
         ))
 
     def post_status(self, content, warning, visibility, in_reply_to_id):
-        data = api.post_status(self.app, self.user, content,
+        data = api.post_status(
+            self.app,
+            self.user,
+            content,
             spoiler_text=warning,
             visibility=visibility,
-            in_reply_to_id=in_reply_to_id)
+            in_reply_to_id=in_reply_to_id
+        ).json()
+
         status = self.make_status(data)
 
         # TODO: fetch new items from the timeline?
@@ -549,6 +605,42 @@ class TUI(urwid.Frame):
         self.footer.set_message("Status posted {} \\o/".format(status.id))
         self.close_overlay()
 
+    def edit_status(self, status, content, warning, visibility, in_reply_to_id):
+        # We don't support editing polls (yet), so to avoid losing the poll
+        # data from the original toot, copy it to the edit request.
+        poll_args = {}
+        poll = status.original.data.get('poll', None)
+
+        if poll is not None:
+            poll_args['poll_options'] = [o['title'] for o in poll['options']]
+            poll_args['poll_multiple'] = poll['multiple']
+
+            # Convert absolute expiry time into seconds from now.
+            expires_at = parse_datetime(poll['expires_at'])
+            expires_in = int((expires_at - datetime.now(timezone.utc)).total_seconds())
+            poll_args['poll_expires_in'] = expires_in
+
+            if 'hide_totals' in poll:
+                poll_args['poll_hide_totals'] = poll['hide_totals']
+
+        data = api.edit_status(
+            self.app,
+            self.user,
+            status.id,
+            content,
+            spoiler_text=warning,
+            visibility=visibility,
+            **poll_args
+        ).json()
+
+        new_status = self.make_status(data)
+
+        self.footer.set_message("Status edited {} \\o/".format(status.id))
+        self.close_overlay()
+
+        if self.timeline is not None:
+            self.timeline.update_status(new_status)
+
     def show_account(self, account_id):
         account = api.whois(self.app, self.user, account_id)
         relationship = api.get_relationship(self.app, self.user, account_id)
