@@ -1,6 +1,6 @@
 from collections import namedtuple
 
-from .utils import parse_datetime
+from toot.utils.datetime import parse_datetime
 
 Author = namedtuple("Author", ["account", "display_name", "username"])
 
@@ -44,17 +44,28 @@ class Status:
         # This can be toggled by the user
         self.show_sensitive = False
 
+        # Set when status is translated
+        self.show_translation = False
+        self.translation = None
+        self.translated_from = None
+
         # TODO: clean up
         self.id = self.data["id"]
         self.account = self._get_account()
         self.created_at = parse_datetime(data["created_at"])
+        if data["edited_at"]:
+            self.edited_at = parse_datetime(data["edited_at"])
+        else:
+            self.edited_at = None
         self.author = self._get_author()
         self.favourited = data.get("favourited", False)
         self.reblogged = data.get("reblogged", False)
+        self.bookmarked = data.get("bookmarked", False)
         self.in_reply_to = data.get("in_reply_to_id")
         self.url = data.get("url")
         self.mentions = data.get("mentions")
         self.reblog = self._get_reblog()
+        self.visibility = data.get("visibility")
 
     @property
     def original(self):
